<?php declare(strict_types=1);

namespace KevinPlugin\Client;

use KevinPlugin\PluginConfig;
use KevinPlugin\Gateway\BankPaymentGatewaySettings;
use KevinPlugin\Utils\Locale;
use KevinPlugin\Utils\Url;
use KevinPluginFramework\Application\Exception\KevinInitException;
use KevinPluginFramework\Application\Exception\KevinRequestException;
use KevinPluginFramework\Application\PaymentClient;
use KevinPluginFramework\Application\PaymentClientSettings;

defined('ABSPATH') || exit;

class GatewayClient
{
    /**
     * @var PaymentClient
     */
    private $kevinClient;

    /**
     * @var array|false
     * false if not initiated
     */
    private static $kevinProjectSettingsCache = false;

    public function __construct(PaymentClient $kevinClient)
    {
        $this->kevinClient = $kevinClient;
    }

    /**
     * @param BankPaymentGatewaySettings $gatewaySettings
     *
     * @return GatewayClient
     *
     * @throws KevinInitException
     */
    public static function init(BankPaymentGatewaySettings $gatewaySettings)
    {
        $clientSettings = new PaymentClientSettings(
            PluginConfig::PLUGIN_VERSION(),
            'Wordpress/WooCommerce',
            $GLOBALS['wp_version'] . '/' . WC_VERSION,
            PluginConfig::KEVIN_API_DOMAIN(),
            Locale::getLocaleLangShortcut()
        );
        $client = new PaymentClient($gatewaySettings->getClientId(), $gatewaySettings->getClientSecret(), $clientSettings);

        return new self($client);
    }

    /**
     * @throws KevinRequestException
     */
    public function initPayment(array $attr): array
    {
        return $this->kevinClient->initPayment($attr);
    }

    /**
     * @throws KevinRequestException
     */
    public function initPaymentRefund(string $paymentId, array $attr): array
    {
        return $this->kevinClient->initiatePaymentRefund($paymentId, $attr);
    }

    /**
     * @throws KevinRequestException
     */
    public function getPayment(string $paymentId): array
    {
        return $this->kevinClient->getPayment($paymentId);
    }

    public function getKevinProjectSettings(): array
    {
        if (self::$kevinProjectSettingsCache === false) {
            try {
                self::$kevinProjectSettingsCache = $this->kevinClient->getProjectSettings();
            } catch (KevinRequestException $e) {
                self::$kevinProjectSettingsCache = [];
            }
        }

        return self::$kevinProjectSettingsCache;
    }

    public function resetKevinProjectSettingsCache()
    {
        self::$kevinProjectSettingsCache = false;
    }

    public function isSandbox(): bool
    {
        return !empty($this->getKevinProjectSettings()['isSandbox']);
    }

    public function isBankPaymentEnabled(): bool
    {
        if (empty($this->getKevinProjectSettings()['paymentMethods'])) {
            return false;
        }

        return in_array('bank', $this->getKevinProjectSettings()['paymentMethods']);
    }

    public function isCardPaymentEnabled(): bool
    {
        if (empty($this->getKevinProjectSettings()['paymentMethods'])) {
            return false;
        }

        return in_array('card', $this->getKevinProjectSettings()['paymentMethods']);
    }

    public function isRefundAllowed(): bool
    {
        return !empty($this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    /**
     * @return bool
     */
    public function isRefundByBankEnabled(): bool
    {
        if (empty($this->getKevinProjectSettings()['allowedRefundsFor'])) {
            return false;
        }

        return in_array('bank', $this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    public function isRefundByCardEnabled(): bool
    {
        if (empty($this->getKevinProjectSettings()['allowedRefundsFor'])) {
            return false;
        }

        return in_array('card', $this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    public function getKevinBanks(bool $withWhiteLogos = false): array
    {
        try {
            $banks = $this->kevinClient->getAllBanks();
        } catch (KevinRequestException $e) {
            return [];
        }

        if (!$withWhiteLogos) {
            return $banks;
        }

        $i = 0;
        foreach ($banks as &$bank) {
            $i++;
            $whiteImageUri = str_replace('/banks/images/', '/banks/images/white/', $bank['imageUri']);
            if ($i === 1 && !Url::isUrlAccessible($whiteImageUri)) {
                break;
            }
            $bank['imageUri'] = $whiteImageUri;
        }

        return $banks;
    }

    public function getKevinBanksByCountry($countryCode): array
    {
        try {
            return $this->kevinClient->getBanks($countryCode);
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    public function getKevinBank($bankId): array
    {
        try {
            return $this->kevinClient->getBank($bankId);
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    public function getKevinCountries(): array
    {
        try {
            return $this->kevinClient->getSupportedCountryCodes();
        } catch (KevinRequestException $e) {
            return [];
        }
    }
}