<?php declare(strict_types=1);

namespace KevinPlugin\Client;

use KevinPlugin\PluginConfig;

defined('ABSPATH') || exit;

class MonitoringClient
{
    const EVENT_INSTALLED = 'installed';
    const EVENT_UNINSTALLED = 'uninstalled';
    const EVENT_ACTIVATED = 'activated';
    const EVENT_DEACTIVATED = 'deactivated';
    const EVENT_UPDATED = 'updated';
    const EVENT_SETTINGS_CHANGED = 'settings_changed';

    /**
     * @var string
     */
    private $url;
    /**
     * @var string
     */
    private $authKey;
    /**
     * @var string
     */
    private $installationId;
    /**
     * @var string|null
     */
    private $clientId;
    /**
     * @var bool
     */
    private $enabled;

	private $settings;

    public function __construct()
    {
		try {
			if ($this->url = PluginConfig::KEVIN_MONITORING_SERVICE_URL()) {
				$this->url = rtrim($this->url, '/') . '/';
			}
			$this->authKey = PluginConfig::KEVIN_MONITORING_SERVICE_KEY();
			$this->enabled = $this->url && $this->authKey;
			$installationIdOptionKey = 'woocommerce_getkevin_installation_id';
			$this->installationId = get_option($installationIdOptionKey, null);
			if (!$this->installationId) {
				$installationId = uniqid('', true);
				update_option($installationIdOptionKey, $installationId);
				$this->installationId = $installationId;
			}
			$this->settings = get_option('woocommerce_getkevin_settings');
			$this->clientId = $this->settings['client_id'] ?? null;
		} catch (\Throwable $e) {
			$this->enabled = false;
		}
    }


    public function sendEvent($type)
    {
        if (!$this->enabled) {
            return;
        }

        $data = [
            'type' => $type,
            'shop' => [
                'url' => home_url('/'),
                'systemType' => 'php',
                'systemVersion' => phpversion(),
                'platformName' => 'Wordpress/WooCommerce',
                'platformVersion' => $GLOBALS['wp_version'] . '/' . (defined('WC_VERSION') ? WC_VERSION : '')
            ],
            'integration' => [
                'version' => PluginConfig::PLUGIN_VERSION(),
                'installationId' => $this->installationId,
                'clientId' => $this->clientId,
                'metadata' => ['theme' => get_stylesheet()]
            ],
            'occurredAt' => microtime(true) * 1000
        ];

        if ($this->clientId) {
            $data['integration']['settings'] = $this->getPluginSettings();
        }

        $this->sendPostRequest('events', $data);
    }

    private function getPluginSettings()
    {
        return [
            'enabled' => ($this->settings['enabled'] ?? null) === 'yes',
            'checkoutView' => $this->settings['checkout_view'] ?? null,
            'countryDropdown' => ($this->settings['country_dropdown'] ?? null) === 'yes',
            'bankSearch' => ($this->settings['bank_search'] ?? null) === 'yes',
            'bankWhiteLogos' => ($this->settings['bank_white_logos'] ?? null) === 'yes',
        ];
    }

    private function sendPostRequest($route, array $data)
    {
        $options = [
            'http' => [
                'header'  => "Content-Type: application/json\r\n" .
                    "X-Kevin-System-Secret: {$this->authKey}\r\n",
                'method'  => 'POST',
                'timeout' => 5,
                'content' => json_encode($data)
            ]
        ];

        $context = stream_context_create($options);
        $result = @file_get_contents($this->url.$route, false, $context);

        if ($result === false) {
            return [
                'isSuccess' => false,
                'data'   => null
            ];
        }

        return [
            'isSuccess' => true,
            'data'   => json_decode($result)
        ];
    }
}