<?php

declare(strict_types=1);

namespace KevinPlugin\Controller;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Gateway\BankPaymentGateway;
use KevinPlugin\Gateway\BankPaymentGatewaySettings;
use KevinPlugin\Gateway\CardPaymentGateway;
use KevinPlugin\GatewaySettings;
use KevinPlugin\Handler\PaymentWebhookHandler;
use KevinPlugin\Handler\RefundWebhookHandler;
use KevinPlugin\Repository\OrderRepository;
use KevinPlugin\Utils\KevinInfoMessage;
use KevinPlugin\Utils\Translator;
use KevinPlugin\Utils\Url;
use KevinPluginFramework\Application\KevinConstants;
use KevinPluginFramework\Application\PaymentWebhookVerifier;

defined('ABSPATH') || exit;

class ApiController
{
    /**
     * @var OrderRepository
     */
    private $orders;
    /**
     * @var GatewaySettings
     */
    private $gatewaySettings;
    /**
     * @var GatewayClient
     */
    private $gatewayClient;

    public function __construct(OrderRepository $orders, BankPaymentGatewaySettings $gatewaySettings, GatewayClient $gatewayClient)
    {
        $this->orders = $orders;
        $this->gatewaySettings = $gatewaySettings;
        $this->gatewayClient = $gatewayClient;
    }

    public function webhook()
    {
        $requestJson = file_get_contents('php://input');
        $request = json_decode($requestJson, true);

        if (!$request) {
            status_header(400);
            exit('Empty request');
        }

        $webhookType = sanitize_text_field($request['type']);

        switch ($webhookType) {
            case KevinConstants::WEBHOOK_TYPE_PAYMENT:
                $paymentId = sanitize_text_field($request['id']);
                $webhookHandler = new PaymentWebhookHandler($this->gatewayClient);
                break;
            case KevinConstants::WEBHOOK_TYPE_REFUND:
                $paymentId = sanitize_text_field($request['paymentId']);
                $webhookHandler = new RefundWebhookHandler();
                break;
            default:
                status_header(400);
                exit('Unrecognized webhook type');
        }

        $order = $this->orders->findOneByPaymentId($paymentId);

        $webhookVerifier = new PaymentWebhookVerifier($this->gatewaySettings->getEndpointSecret(), Url::generateWebhookUrl());
        if (!$webhookVerifier->verifySignature(json_encode($request), getallheaders())) {
            if ($order) {
                $order->add_order_note(KevinInfoMessage::withPrefix(Translator::translate('Unable to change order status. Please check whether Endpoint Secret is correct.')));
            }
            status_header(400);
            exit('Signatures do not match.');
        }

        if (!$order) {
            status_header(400);
            exit('Order not found');
        }

        try {
            $result = $webhookHandler->handle($order, $request);
        } catch (\Throwable $e) {
            status_header(500);
            exit($e->getMessage());
        }

        if ($result['isSuccess'] === true) {
            status_header(200);
        } else {
            status_header(400);
        }

        exit($result['message']);
    }

    public function return()
    {
        $statusGroup = sanitize_text_field($_GET['statusGroup']);
        $kevinPaymentId = sanitize_text_field($_GET['paymentId']);
        $kevinReturnToken = sanitize_text_field($_GET['kv_wc_tkn']);

        $order = $this->orders->findOneByPaymentId($kevinPaymentId);

        if (!$order || !in_array($order->get_payment_method(), [BankPaymentGateway::GATEWAY_ID, CardPaymentGateway::GATEWAY_ID])) {
            status_header(400);
            exit();
        }

        if (!$kevinReturnToken || $order->get_meta('_kevin_return_token') !== $kevinReturnToken) {
            status_header(400);
            exit();
        }

        $order->add_order_note(KevinInfoMessage::withPrefix(Translator::translate('Customer returned to the store.')));

        if (in_array($statusGroup, [KevinConstants::STATUS_GROUP_PENDING, KevinConstants::STATUS_GROUP_COMPLETED])) {
            $this->orders->updateOrderMetadataKevinReturnStatusGroup($order, $statusGroup);
            $redirectUrl = add_query_arg(['statusGroup' => $statusGroup], $order->get_checkout_order_received_url());
        } else {
            if ($statusGroup === KevinConstants::STATUS_GROUP_FAILED) {
                wc_add_notice(Translator::translate('Your order cannot be processed because the bank declined your transaction. Please try again.'), 'error');
            } else {
                wc_add_notice(Translator::translate('Payment initiation was cancelled.'), 'error');
            }
            $redirectUrl = wc_get_checkout_url();
        }

        wp_redirect($redirectUrl);
        exit();
    }
}