<?php

declare(strict_types=1);

namespace KevinPlugin\Form\Settings;

use KevinPlugin\Form\Settings\Subform\CheckoutSettingsSubform;
use KevinPlugin\Form\Settings\Subform\MainSettingsSubform;
use KevinPlugin\Form\Settings\Subform\SubformInterface;
use KevinPlugin\Gateway\BankPaymentGatewaySettings;
use KevinPlugin\Utils\Translator;
use KevinPluginFramework\Presentation\Admin\Form\BuildGuideline\CheckoutSettingsFormBuildGuideline;
use KevinPluginFramework\Presentation\Admin\Form\BuildGuideline\MainSettingsFormBuildGuideline;
use KevinPluginFramework\Presentation\Admin\Form\Validator\IbanRegexValidator;
use KevinPluginFramework\Presentation\Admin\Form\Validator\ValidatorInterface;

defined('ABSPATH') || exit;

class BankPaymentsSettingsForm
{
	/**
	 * @var array
	 */
	private $formFields;

	/**
	 * @var SubformInterface[]
	 */
	private $subForms;

	public function __construct(BankPaymentGatewaySettings $gatewaySettings)
	{
		$this->subForms = [
			new MainSettingsSubform(new MainSettingsFormBuildGuideline()),
			new CheckoutSettingsSubform(new CheckoutSettingsFormBuildGuideline(), $gatewaySettings)
		];

		$this->initFormFields();
	}

	public function getFormFields(): array
	{
		return $this->formFields;
	}

	/**
	 * @return array|SubformInterface[]
	 */
	public function getSubForms(): array
	{
		return $this->subForms;
	}

	private function initFormFields()
	{
		$this->formFields = [];
		foreach ($this->subForms as $subForm) {
			$this->formFields = array_merge($this->formFields, $subForm->getFormFields());
		}
	}

	public function isValid($data): bool
	{
		$isValid = true;

		foreach ($data as $settingKey => $settingValue) {

			$formField = $this->getFormFields()[$settingKey];

			if (isset($formField['custom_attributes']['required']) && !$settingValue) {
				$isValid = false;
				\WC_Admin_Settings::add_error(Translator::translate($formField['title']) . ': ' . Translator::translate('This field is required.'));
				continue;
			}

			if (isset($formField['custom_validators'])) {
				/** @var ValidatorInterface $validator */
				foreach ($formField['custom_validators'] as $validator) {
					if (!$validator->validate($settingValue)) {
						$isValid = false;
						if ($validator instanceof IbanRegexValidator) {
							$message = sprintf(Translator::translate($validator->getMessage()), Translator::translate('LT01234567891011121314'));
						} else {
							$message = Translator::translate($validator->getMessage());
						}
						\WC_Admin_Settings::add_error(Translator::translate($formField['title']) . ': ' . $message);
					}
				}
			}
		}

		return $isValid;
	}
}
