<?php declare(strict_types=1);

namespace KevinPlugin\Gateway;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Client\MonitoringClient;
use KevinPlugin\Controller\ApiController;
use KevinPlugin\Form\Settings\BankPaymentsSettingsForm;
use KevinPlugin\Form\Settings\Subform\CheckoutSettingsSubform;
use KevinPlugin\Handler\PaymentHandler;
use KevinPlugin\Handler\RefundHandler;
use KevinPlugin\Repository\OrderRepository;
use KevinPlugin\Utils\Translator;
use KevinPlugin\Utils\Url;
use KevinPlugin\View\BankPaymentSettingsFormView;
use KevinPlugin\View\BankPaymentView;
use KevinPlugin\View\PaymentLegalInfoView;
use KevinPlugin\View\PaymentPrependView;
use KevinPlugin\WcConstants;

defined('ABSPATH') || exit;

class BankPaymentGateway extends AbstractPaymentGateway
{
    const GATEWAY_ID = 'getkevin';

    /**
     * @var BankPaymentGatewaySettings
     */
    private $gatewaySettings;
    /**
     * @var GatewayClient|null
     */
    private $gatewayClient;
    /**
     * @var BankPaymentsSettingsForm
     */
    private $settingsForm;
    /**
     * @var OrderRepository
     */
    private $orders;
    /**
     * @var ApiController
     */
    private $apiController;

    public function __construct()
    {
        $this->id = self::GATEWAY_ID;
        $this->method_title = 'kevin.';
        $this->method_description = Translator::translate('Let your customers make fast, simple and secure payments directly from their bank accounts across Europe.');
        $this->title = Translator::translate('Bank payments');
        $this->supports = [WcConstants::WC_FEATURE_PRODUCTS, WcConstants::WC_FEATURE_REFUNDS];

        $this->init_settings();
        $this->settingsForm = new BankPaymentsSettingsForm($this->gatewaySettings);
        $this->init_form_fields();

        $this->orders = new OrderRepository();

        if ($this->gatewaySettings->getClientId() && $this->gatewaySettings->getClientSecret() && $this->gatewaySettings->getEndpointSecret()) {
            $this->gatewayClient = GatewayClient::init($this->gatewaySettings);
        }

        if ($this->gatewayClient) {
            $this->apiController = new ApiController($this->orders, $this->gatewaySettings, $this->gatewayClient);
            /**
             * @see Url::generateWebhookUrl()
             * @see ApiController::webhook()
             */
            add_action('woocommerce_api_wc_gateway_' . $this->id, [$this->apiController, 'webhook']);
            /**
             * @see Url::generateReturnUrl()
             * @see ApiController::return()
             */
            add_action('woocommerce_api_kevin_return', [$this->apiController, 'return']);
        }

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        add_filter('woocommerce_thankyou_order_received_text', [$this, 'render_thankyou_order_received_text'], 10, 2);
    }

    public function is_available(): bool
    {
        if (!is_checkout()) {
            return parent::is_available();
        }

        return parent::is_available() && $this->isGatewayReady() && $this->gatewayClient->isBankPaymentEnabled();
    }

    public function has_fields(): bool
    {
        if ($this->gatewaySettings->isCheckoutViewFrame() && !$this->gatewayClient->isSandbox()) {
            return false;
        }

        return true;
    }

    public function needs_setup(): bool
    {
        return true;
    }

    public function init_settings()
    {
        parent::init_settings();
        $this->gatewaySettings = new BankPaymentGatewaySettings($this->settings);
    }

    public function init_form_fields()
    {
        $this->form_fields = $this->settingsForm->getFormFields();
    }

    public function admin_options()
    {
        $activeFormId = sanitize_text_field($_GET['kevin-tab'] ?? CheckoutSettingsSubform::FORM_ID);
        $view = new BankPaymentSettingsFormView($this->settingsForm, $this, $this->gatewayClient, $activeFormId);
        $view->render();
    }

    public function process_admin_options()
    {
        if (!$this->settingsForm->isValid($this->extractSettingsFormData())) {
            return false;
        }

        parent::process_admin_options();

        if ($this->gatewayClient) {
            $this->gatewayClient->resetKevinProjectSettingsCache();
        }

        (new MonitoringClient())->sendEvent(MonitoringClient::EVENT_SETTINGS_CHANGED);
    }

    public function payment_fields()
    {
        (new PaymentPrependView($this->gatewayClient))->render();

        if (!$this->gatewaySettings->isCheckoutViewFrame()) {
            (new BankPaymentView($this->gatewayClient, $this->gatewaySettings))->render();
            (new PaymentLegalInfoView())->render();
        }
    }

    public function validate_fields(): bool
    {
        if (!$this->gatewaySettings->isCheckoutViewFrame() && empty($_REQUEST['payment']['bank_id'])) {
            wc_add_notice(Translator::translate('Please select your bank.'), 'error');

            return false;
        }

        return true;
    }

    public function process_payment($order_id): array
    {
        $order = $this->orders->findByOrderId($order_id);
        $bankId = isset($_REQUEST['payment']['bank_id']) ? sanitize_text_field($_REQUEST['payment']['bank_id']) : null;
        $paymentHandler = new PaymentHandler($this->gatewayClient, $this->gatewaySettings, $this->orders);
        return $paymentHandler->handle($order, $bankId);
    }

    public function process_refund($order_id, $amount = null, $reason = '')
    {
        $order = $this->orders->findByOrderId($order_id);
        $refundHandler = new RefundHandler($this->gatewayClient);
        return $refundHandler->handle($order, (float)$amount, (string)$reason);
    }

    public function can_refund_order($order): bool
    {
        if ($order->get_payment_method() !== $this->id || !$order->get_date_paid() || !$order->get_remaining_refund_amount()) {
            return false;
        }
        if (!$this->gatewayClient->isRefundByBankEnabled()) {
            return false;
        }

        return parent::can_refund_order($order);
    }

    public function isGatewayReady(): bool
    {
        return $this->gatewayClient && $this->gatewayClient->getKevinProjectSettings();
    }
}