<?php declare(strict_types=1);

namespace KevinPlugin\Gateway;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Handler\PaymentHandler;
use KevinPlugin\Handler\RefundHandler;
use KevinPlugin\Repository\OrderRepository;
use KevinPlugin\Utils\Translator;
use KevinPlugin\Utils\Url;
use KevinPlugin\View\CardPaymentSettingsFormView;
use KevinPlugin\View\PaymentPrependView;
use KevinPlugin\WcConstants;

defined('ABSPATH') || exit;

class CardPaymentGateway extends AbstractPaymentGateway
{
    const GATEWAY_ID = 'kevin_card';

    const CARD_PAYMENT_KEY = 'card';

    /**
     * @var GatewayClient|null
     */
    private $gatewayClient;
    /**
     * @var OrderRepository
     */
    private $orders;
    /**
     * @var BankPaymentGatewaySettings
     */
    private $bankPaymentGatewaySettings;

    public function __construct()
    {
        $this->id = self::GATEWAY_ID;
        $this->method_title = 'kevin.';
        $this->method_description = 'kevin. '.Translator::translate('Card payments').'.';
        $this->title = Translator::translate('Card payments');
        $this->icon = Url::generateUrlToFile('assets/images/VISA_MC_small.png');
        $this->supports = [WcConstants::WC_FEATURE_PRODUCTS, WcConstants::WC_FEATURE_REFUNDS];

        $this->init_settings();
        $this->init_form_fields();

        $this->bankPaymentGatewaySettings = new BankPaymentGatewaySettings(get_option('woocommerce_getkevin_settings', []));
        if ($this->bankPaymentGatewaySettings->getClientId() &&
            $this->bankPaymentGatewaySettings->getClientSecret() &&
            $this->bankPaymentGatewaySettings->getEndpointSecret()) {
            $this->gatewayClient = GatewayClient::init($this->bankPaymentGatewaySettings);
        }

        $this->orders = new OrderRepository();

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        add_filter('woocommerce_thankyou_order_received_text', [$this, 'render_thankyou_order_received_text'], 10, 2);
    }

    public function is_available(): bool
    {
        if (!is_checkout()) {
            return parent::is_available();
        }

        return parent::is_available() && $this->isGatewayReady() && $this->gatewayClient->isCardPaymentEnabled();
    }

    public function has_fields(): bool
    {
        return $this->gatewayClient->isSandbox();
    }

    public function init_form_fields()
    {
        $this->form_fields = [
            'enabled' => [
                'title'   => Translator::translate('Enable / Disable'),
                'type'    => 'checkbox',
                'label'   => ' ',
                'default' => 'no',
            ],
        ];
    }

    public function admin_options()
    {
        $view = new CardPaymentSettingsFormView($this, $this->gatewayClient);
        $view->render();
    }

    public function payment_fields()
    {
        (new PaymentPrependView($this->gatewayClient))->render();
    }

    public function process_payment($order_id): array
    {
        $order = $this->orders->findByOrderId($order_id);
        $paymentHandler = new PaymentHandler($this->gatewayClient, $this->bankPaymentGatewaySettings, $this->orders);
        return $paymentHandler->handle($order, self::CARD_PAYMENT_KEY);
    }

    public function process_refund($order_id, $amount = null, $reason = '')
    {
        $order = $this->orders->findByOrderId($order_id);
        $refundHandler = new RefundHandler($this->gatewayClient);
        return $refundHandler->handle($order, (float)$amount, (string)$reason);
    }

    public function can_refund_order($order): bool
    {
        if ($order->get_payment_method() !== $this->id || !$order->get_date_paid() || !$order->get_remaining_refund_amount()) {
            return false;
        }
        if (!$this->gatewayClient->isRefundByCardEnabled()) {
            return false;
        }

        return parent::can_refund_order($order);
    }

    public function isGatewayReady(): bool
    {
        return $this->gatewayClient && $this->gatewayClient->getKevinProjectSettings();
    }
}