<?php

declare(strict_types=1);

namespace KevinPlugin\Handler;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Gateway\BankPaymentGatewaySettings;
use KevinPlugin\Gateway\CardPaymentGateway;
use KevinPlugin\Repository\OrderRepository;
use KevinPlugin\Utils\KevinInfoMessage;
use KevinPlugin\Utils\Translator;
use KevinPlugin\Utils\Url;
use KevinPlugin\WcConstants;

defined('ABSPATH') || exit;

class PaymentHandler
{
	/**
	 * @var BankPaymentGatewaySettings
	 */
	private $gatewaySettings;
    /**
     * @var GatewayClient
     */
    private $gateway;
    /**
     * @var OrderRepository
     */
    private $orderRepository;

    public function __construct(GatewayClient $gateway, BankPaymentGatewaySettings $gatewaySettings, OrderRepository $orderRepository)
	{
        $this->gateway = $gateway;
        $this->gatewaySettings = $gatewaySettings;
        $this->orderRepository = $orderRepository;
    }

	public function handle(\WC_Order $order, $bankId): array
	{
		try {
            $order->add_order_note(KevinInfoMessage::withPrefix(Translator::translate('Payment started.')));

            $returnUrlToken = uniqid();
			$attr = $this->createKevinPaymentAttr($order, Url::generateWebhookUrl(), Url::generateReturnUrl($returnUrlToken), $bankId);
			$response = $this->gateway->initPayment($attr);

			$this->orderRepository->updateOrderWithKevinMetadata($order, $response, $returnUrlToken);
			$order->add_order_note(KevinInfoMessage::withPrefix(sprintf(Translator::translate('Payment ID is: %s'), $response['id'])));

			return [
				'result' => 'success',
				'redirect' => $response['confirmLink'],
			];
		} catch (\Exception $e) {

			$order->update_status(WcConstants::WC_ORDER_STATUS_CANCELLED, KevinInfoMessage::withPrefix(sprintf('Your payment could not be created: %s', $e->getMessage())));

			return [
				'result' => 'fail',
				'redirect' => ''
			];
		}
	}

	private function createKevinPaymentAttr(\WC_Order $order, string $webhookUrl, string $redirectUrl, $bankId): array
	{
		$attr = [
			'amount' => number_format((float)$order->get_total(), 2, '.', ''),
			'currencyCode' => $order->get_currency(),
			'description' => sprintf('%1$03d', (string)$order->get_id()),
			'identifier' => ['email' => $order->get_billing_email()],
			'redirectPreferred' => true,
			'Redirect-URL' => $redirectUrl,
			'Webhook-URL' => $webhookUrl,
		];

		$attr['bankPaymentMethod'] = [
			'creditorName' => $this->gatewaySettings->getCreditorName(),
			'endToEndId' => (string)$order->get_id(),
			'creditorAccount' => [
				'iban' => $this->gatewaySettings->getCreditorAccount(),
			],
		];

        if ($bankId === CardPaymentGateway::CARD_PAYMENT_KEY) { //card
            $attr['cardPaymentMethod'] = [];
            $attr['paymentMethodPreferred'] = 'card';
        } elseif ($bankId) { //bank
            $attr['bankId'] = $bankId;
        } else { //frame
            $attr['paymentMethodPreferred'] = 'bank';
        }

		return $attr;
	}
}
