<?php declare(strict_types=1);

namespace KevinPlugin\Handler;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Gateway\BankPaymentGateway;
use KevinPlugin\Gateway\CardPaymentGateway;
use KevinPlugin\Utils\KevinInfoMessage;
use KevinPlugin\Utils\Translator;
use KevinPlugin\WcConstants;
use KevinPluginFramework\Application\KevinConstants;

defined('ABSPATH') || exit;

class PaymentWebhookHandler implements WebhookHandlerInterface
{
    /**
     * @var GatewayClient
     */
    private $gatewayClient;

    public function __construct(GatewayClient $gatewayClient)
    {
        $this->gatewayClient = $gatewayClient;
    }

    public function handle(\WC_Order $order, array $request): array
    {
        if (!in_array($order->get_payment_method(), [BankPaymentGateway::GATEWAY_ID, CardPaymentGateway::GATEWAY_ID])) {
            return [
                'isSuccess' => true,
                'message' => 'Order paid with different payment method'
            ];
        }

        if ($order->has_status(wc_get_is_paid_statuses())) {
            return [
                'isSuccess' => true,
                'message' => 'Order already paid'
            ];
        }

        if ($order->has_status(WcConstants::WC_ORDER_STATUS_CANCELLED)) {
            return [
                'isSuccess' => true,
                'message' => 'Order cancelled'
            ];
        }

        $paymentId = $request['id'];
        $statusGroup = $request['statusGroup'];
        $status = $request['bankStatus'] ?? $request['cardStatus'] ?? $request['hybridStatus'];
        $returnStatusGroup = $order->get_meta('_kevin_return_status_group');

        $order->update_meta_data('_kevin_status', $status);
        $order->update_meta_data('_kevin_status_group', $statusGroup);

        if ($statusGroup === KevinConstants::STATUS_GROUP_COMPLETED) {

            $kevinPayment = $this->gatewayClient->getPayment($paymentId);

            if (round((float)$kevinPayment['amount'], 2) !== round((float)$order->get_total(), 2)) {
                $order->update_status(WcConstants::WC_ORDER_STATUS_ON_HOLD, 'kevin. payment confirmation received but amount payed is not equal to order amount!');

                return [
                    'isSuccess' => false,
                    'message' => 'kevin. payment amount not equal to order amount!'
                ];
            }

            $order->payment_complete($paymentId);
        } elseif ($statusGroup === KevinConstants::STATUS_GROUP_FAILED && in_array($returnStatusGroup, [KevinConstants::STATUS_GROUP_PENDING, KevinConstants::STATUS_GROUP_COMPLETED])) {
            $order->update_status(WcConstants::WC_ORDER_STATUS_FAILED, KevinInfoMessage::withPrefix(sprintf(Translator::translate('Payment status is: %s').'<br>', $statusGroup)));
        }

        return [
            'isSuccess' => true,
            'message' => 'Signatures match.'
        ];
    }
}