<?php

declare(strict_types=1);

namespace KevinPlugin\View;

use KevinPlugin\Client\GatewayClient;
use KevinPlugin\Form\Settings\Subform\CheckoutSettingsSubform;
use KevinPlugin\Gateway\CardPaymentGateway;
use KevinPlugin\PluginConfig;
use KevinPlugin\Form\Settings\BankPaymentsSettingsForm;
use KevinPlugin\Form\Settings\Subform\SubformInterface;
use KevinPlugin\Gateway\BankPaymentGateway;
use KevinPlugin\Utils\Translator;
use KevinPlugin\Utils\Url;

class BankPaymentSettingsFormView extends AbstractSettingsView
{
	const TEMPLATE_PATH = __DIR__.'/../../templates/kevin-banks-settings-form-view.php';

	/**
	 * @var BankPaymentsSettingsForm
	 */
	private $settingsForm;
    /**
     * @var BankPaymentGateway
     */
    private $bankPaymentGateway;
    /**
     * @var GatewayClient|null
     */
    private $gatewayClient;
    /**
     * @var int
     */
    private $activeFormId;

    public function __construct(
        BankPaymentsSettingsForm $settingsForm,
        BankPaymentGateway $bankPaymentGateway,
        $gatewayClient,
        string $activeFormId)
	{
		$this->settingsForm = $settingsForm;
        $this->bankPaymentGateway = $bankPaymentGateway;
        $this->gatewayClient = $gatewayClient;
        $this->activeFormId = $activeFormId;
    }

	public function render()
	{
		include self::TEMPLATE_PATH;
		wp_register_script('kevin-bank-payment-settings', Url::generateUrlToFile('assets/js/backend/kevin-bank-payment-settings.js'), [], PluginConfig::PLUGIN_VERSION());
		wp_enqueue_script('kevin-bank-payment-settings');
	}

    /**
     * @return SubformInterface[]
     */
	protected function getSubForms(): array
	{
		return $this->settingsForm->getSubForms();
	}

    protected function getActiveFormId(): string
    {
        return $this->activeFormId;
    }

	protected function getHtmlForSubForm(SubformInterface $subForm): string
	{
		$formFields = $subForm->getFormFields();
		$this->addAsteriskToRequiredFields($formFields);
		return $this->bankPaymentGateway->generate_settings_html($formFields, false);
	}

    protected function getBankPaymentSettingsUrl(): string
    {
        return admin_url( 'admin.php?page=wc-settings&tab=checkout&section='.BankPaymentGateway::GATEWAY_ID);
    }

    protected function getCardPaymentSettingsUrl(): string
    {
        return admin_url( 'admin.php?page=wc-settings&tab=checkout&section='.CardPaymentGateway::GATEWAY_ID);
    }

	protected function getInfoMessages(): array
	{
		if (!$this->bankPaymentGateway->isGatewayReady()) {
			return [];
		}

        if ($this->activeFormId === CheckoutSettingsSubform::FORM_ID) {
            $shopSettingMessage = $this->boldHtmlText(Translator::translate('Your shop settings from kevin. dashboard:')) . ' ';
            if ($this->gatewayClient->isBankPaymentEnabled()) {
                $shopSettingMessage .= Translator::translate('Bank payment method is available.');
            } else {
                $shopSettingMessage .= Translator::translate('Bank payment method is not available.');
            }
            $shopSettingMessage .= ' ';
            if ($this->gatewayClient->isRefundByBankEnabled()) {
                $shopSettingMessage .= Translator::translate('Bank refunds are allowed.');
            } else {
                $shopSettingMessage .= Translator::translate('Bank refunds are not allowed.');
            }

            return [$shopSettingMessage];
        }

		return [];
	}

	protected function getWarningMessages(): array
	{
		if (!$this->bankPaymentGateway->isGatewayReady()) {
			return [];
		}

		$messages = [];

		if ($this->gatewayClient->isSandbox()) {
			$messages[] = Translator::translate('kevin. payment gateway is in test mode for testing purposes. Real payments are not available.');
		}

		return $messages;
	}

    protected function getErrorMessages(): array
    {
        if (!$this->gatewayClient) {
            return [];
        }

        $messages = [];

        if (!$this->gatewayClient->getKevinProjectSettings()) {
            $messages[] = Translator::translate('Incorrect client credentials.');
        }

        return $messages;
    }
}
