<?php

namespace KevinPluginFramework\Application;

use Kevin\Client;
use Kevin\KevinException;
use KevinPluginFramework\Application\Exception\KevinInitException;
use KevinPluginFramework\Application\Exception\KevinRequestBadRequestException;
use KevinPluginFramework\Application\Exception\KevinRequestException;
use KevinPluginFramework\Application\Exception\KevinRequestInternalServerException;
use KevinPluginFramework\Application\Exception\KevinRequestUnauthorizedException;

class PaymentClient
{
    /**
     * @var Client
     */
    private $client;

    /**
     * @param string $clientId
     * @param string $clientSecret
     * @param PaymentClientSettings $settings
     * @throws KevinInitException
     */
    public function __construct($clientId, $clientSecret, $settings)
    {
		if (!isset($clientId, $clientSecret)) {
			throw new KevinInitException('missing credentials');
		}

        if (!$settings instanceof PaymentClientSettings) {
			throw new KevinInitException('$settings is of wrong type');
        }

        try {
            $this->client = new Client($clientId, $clientSecret, $settings->toArray());
        } catch (KevinException $e) {
            throw new KevinInitException($e->getMessage());
        }
    }

    /**
     * @param array $attributes
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function initPayment($attributes)
    {
        try {
            return $this->client->payment()->initPayment($attributes);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $paymentId
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     */
    public function getPayment($paymentId)
    {
        try {
            return $this->client->payment()->getPayment($paymentId);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $paymentId
     * @param array $attributes
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function initiatePaymentRefund($paymentId, $attributes)
    {
        try {
            return $this->client->payment()->initiatePaymentRefund($paymentId, $attributes);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $countryCode
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getBanks($countryCode)
    {
        try {
            return $this->client->auth()->getBanks(['countryCode' => $countryCode])['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getAllBanks()
    {
        try {
            return $this->client->auth()->getBanks()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getSupportedCountryCodes()
    {
        try {
            return $this->client->auth()->getCountries()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $bankCode
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getBank($bankCode)
    {
        try {
            return $this->client->auth()->getBank($bankCode);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getPaymentMethods()
    {
        try {
            return $this->client->auth()->getPaymentMethods()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

	/**
	 * @return array
	 * @throws KevinRequestBadRequestException
	 * @throws KevinRequestInternalServerException
	 * @throws KevinRequestUnauthorizedException
	 * @throws KevinRequestException
	 */
	public function getProjectSettings()
	{
		try {
			return $this->client->auth()->getProjectSettings();
		} catch (KevinException $e) {
			throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
		}
	}
}
